/*
   AMVisC++Interface - C++ Interface for Body- and Pos-File Creation
   Used by AMVis
   Copyright (C) 2006 Institute of Applied Mechanics,
   Technical University of Munich

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   */

#ifndef INVISIBLEBODY_H
#define INVISIBLEBODY_H

#include <crigidbody.h>

namespace AMVis {

  //! Creates a invisible body.
  /*!
   * This class is used to create a invisible body, which can be used to 
   * moved and rotation the camera in AMVis.
   */
  class InvisibleBody : public CRigidBody {
    public:
      //! Create a invisible body.
      /*!
       * baseFilename is the basename of the body and pos file. instances is the
       * number of instances this body should create and if binPosFile is true the
       * pos file is created in binary mode or in ascii mode if false.
       */
      InvisibleBody(const std::string &, int, bool);

      //! Write a invisible body file.
      virtual void writeBodyFile() const;

    protected:
      void writeToBodyFile(std::ofstream *) const;
  };

}

#endif
